@@ -1,12 +1,13 @@
 package Test::DBIC::ExpectedQueries::Query;
-$Test::DBIC::ExpectedQueries::Query::VERSION = '1.008';
+$Test::DBIC::ExpectedQueries::Query::VERSION = '1.009';
 use Moo;
 
 
 
-has sql       => ( is => "ro", required => 1 );
-has table     => ( is => "rw" );
-has operation => ( is => "rw" );
+has sql         => ( is => "ro", required => 1 );
+has stack_trace => ( is => "ro", required => 1 ); # Just a string
+has table       => ( is => "rw" );
+has operation   => ( is => "rw" );
 
 sub BUILD { shift->analyze_sql() }
 
@@ -44,4 +45,9 @@ sub display_sql {
     return "SQL: (" . $self->sql . ")";
 }
 
+sub display_stack_trace {
+    my $self = shift;
+    return "     " . $self->stack_trace;
+}
+
 1;
